@@ -16,7 +16,8 @@ class FilterSelect:
     def __init__(self):
         return
 
-    def _filter_F_one_feature(self, data, treatment_indicator, feature_name, y_name): 
+    @staticmethod
+    def _filter_F_one_feature(data, treatment_indicator, feature_name, y_name):
         """
         Conduct F-test of the interaction between treatment and one feature.
 
@@ -79,7 +80,8 @@ class FilterSelect:
         return all_result
 
 
-    def _filter_LR_one_feature(self, data, treatment_indicator, feature_name, y_name, disp=True): 
+    @staticmethod
+    def _filter_LR_one_feature(data, treatment_indicator, feature_name, y_name, disp=True):  
         """
         Conduct LR (Likelihood Ratio) test of the interaction between treatment and one feature.
 
@@ -152,9 +154,10 @@ class FilterSelect:
 
 
     # Get node summary - a function 
-    def _GetNodeSummary(self, data, 
+    @staticmethod
+    def _GetNodeSummary(data,
                         experiment_group_column='treatment_group_key', 
-                        y_name='conversion'):
+                        y_name='conversion', smooth=True):
         """
         To count the conversions and get the probabilities by treatment groups. This function comes from the uplift tree algorithm, that is used for tree node split evaluation.
 
@@ -162,6 +165,13 @@ class FilterSelect:
         ----------
         data : DataFrame
             The DataFrame that contains all the data (in the current "node").  
+        experiment_group_column : str
+            Treatment indicator column name.
+        y_name : str
+            Label indicator column name.
+        smooth : bool
+            Smooth label count by adding 1 in case certain labels do not occur
+            naturally with a treatment. Prevents zero divisions.
 
         Returns
         -------
@@ -188,7 +198,12 @@ class FilterSelect:
         for ti in treatment_group_keys: 
             results.update({ti: {}}) 
             for ci in y_name_keys:
-                results[ti].update({ci: results_series[ti, ci]}) 
+                if smooth:
+                    results[ti].update({ci: results_series[ti, ci]
+                                        if results_series.index.isin([(ti, ci)]).any()
+                                        else 1})
+                else:
+                    results[ti].update({ci: results_series[ti, ci]})
 
         # Probability of conversion and group size by treatment group
         nodeSummary = {}
@@ -202,7 +217,8 @@ class FilterSelect:
         return results, nodeSummary 
 
     # Divergence-related functions, from upliftpy
-    def _kl_divergence(self, pk, qk):
+    @staticmethod
+    def _kl_divergence(pk, qk):
         """
         Calculate KL Divergence for binary classification.
 
@@ -241,7 +257,8 @@ class FilterSelect:
                 d_res += self._kl_divergence(nodeSummary[treatment_group][0], pc)
         return d_res
 
-    def _evaluate_ED(self, nodeSummary, control_group='control'):
+    @staticmethod
+    def _evaluate_ED(nodeSummary, control_group='control'):
         """
         Calculate the multi-treatment unconditional D (one node)
         with Euclidean Distance as split Evaluation function.
@@ -260,7 +277,8 @@ class FilterSelect:
                 d_res += 2 * (nodeSummary[treatment_group][0] - pc)**2
         return d_res
 
-    def _evaluate_Chi(self, nodeSummary, control_group='control'):
+    @staticmethod
+    def _evaluate_Chi(nodeSummary, control_group='control'):
         """
         Calculate the multi-treatment unconditional D (one node)
         with Chi-Square as split Evaluation function.
@@ -318,8 +336,10 @@ class FilterSelect:
             evaluationFunction = self._evaluate_Chi
 
         totalSize = len(data.index)
+
+        # drop duplicate edges in pq.cut result to avoid issues
         x_bin = pd.qcut(data[feature_name].values, n_bins, labels=False, 
-                        duplicates='raise')
+                        duplicates='drop')
         d_children = 0
         for i_bin in range(x_bin.max() + 1): # range(n_bins):
             nodeSummary = self._GetNodeSummary(
