@@ -7,15 +7,17 @@ from scipy.stats import norm
 from sklearn.model_selection import cross_val_predict, KFold, train_test_split
 from xgboost import XGBRegressor
 
-from causalml.inference.meta.utils import (check_treatment_vector, check_p_conditions,
+from causalml.inference.meta.base import BaseLearner
+from causalml.inference.meta.utils import (check_treatment_vector,
     get_xgboost_objective_metric, convert_pd_to_np)
 from causalml.inference.meta.explainer import Explainer
-from causalml.propensity import compute_propensity_score
+from causalml.propensity import compute_propensity_score, ElasticNetPropensityModel
+
 
 logger = logging.getLogger('causalml')
 
 
-class BaseRLearner(object):
+class BaseRLearner(BaseLearner):
     """A parent class for R-learner classes.
 
     An R-learner estimates treatment effects with two machine learning models and the propensity score.
@@ -27,6 +29,7 @@ class BaseRLearner(object):
                  learner=None,
                  outcome_learner=None,
                  effect_learner=None,
+                 propensity_learner=ElasticNetPropensityModel(),
                  ate_alpha=.05,
                  control_name=0,
                  n_fold=5,
@@ -38,22 +41,19 @@ class BaseRLearner(object):
             outcome_learner (optional): a model to estimate outcomes
             effect_learner (optional): a model to estimate treatment effects. It needs to take `sample_weight` as an
                 input argument for `fit()`
+            propensity_learner (optional): a model to estimate propensity scores. `ElasticNetPropensityModel()` will
+                be used by default.
             ate_alpha (float, optional): the confidence level alpha of the ATE estimate
             control_name (str or int, optional): name of control group
             n_fold (int, optional): the number of cross validation folds for outcome_learner
             random_state (int or RandomState, optional): a seed (int) or random number generator (RandomState)
         """
         assert (learner is not None) or ((outcome_learner is not None) and (effect_learner is not None))
+        assert propensity_learner is not None
 
-        if outcome_learner is None:
-            self.model_mu = deepcopy(learner)
-        else:
-            self.model_mu = outcome_learner
-
-        if effect_learner is None:
-            self.model_tau = deepcopy(learner)
-        else:
-            self.model_tau = effect_learner
+        self.model_mu = outcome_learner if outcome_learner else deepcopy(learner)
+        self.model_tau = effect_learner if outcome_learner else deepcopy(learner)
+        self.model_p = propensity_learner
 
         self.ate_alpha = ate_alpha
         self.control_name = control_name
@@ -65,10 +65,10 @@ class BaseRLearner(object):
         self.propensity_model = None
 
     def __repr__(self):
-        return ('{}(model_mu={},\n'
-                '\tmodel_tau={})'.format(self.__class__.__name__,
-                                         self.model_mu.__repr__(),
-                                         self.model_tau.__repr__()))
+        return (f'{self.__class__.__name__}\n'
+                f'\toutcome_learner={self.model_mu.__repr__()}\n'
+                f'\teffect_learner={self.model_tau.__repr__()}\n'
+                f'\tpropensity_learner={self.model_p.__repr__()}')
 
     def fit(self, X, treatment, y, p=None, verbose=True):
         """Fit the treatment effect and outcome models of the R learner.
@@ -88,27 +88,10 @@ class BaseRLearner(object):
         self.t_groups.sort()
 
         if p is None:
-            logger.info('Generating propensity score')
-            p = dict()
-            p_model = dict()
-            for group in self.t_groups:
-                mask = (treatment == group) | (treatment == self.control_name)
-                treatment_filt = treatment[mask]
-                X_filt = X[mask]
-                w_filt = (treatment_filt == group).astype(int)
-                w = (treatment == group).astype(int)
-                p[group], p_model[group] = compute_propensity_score(X=X_filt, treatment=w_filt,
-                                                                    X_pred=X, treatment_pred=w)
-            self.propensity_model = p_model
-            self.propensity = p
+            self._set_propensity_models(X=X, treatment=treatment, y=y)
+            p = self.propensity
         else:
-            check_p_conditions(p, self.t_groups)
-
-        if isinstance(p, (np.ndarray, pd.Series)):
-            treatment_name = self.t_groups[0]
-            p = {treatment_name: convert_pd_to_np(p)}
-        elif isinstance(p, dict):
-            p = {treatment_name: convert_pd_to_np(_p) for treatment_name, _p in p.items()}
+            p = self._format_p(p, self.t_groups)
 
         self._classes = {group: i for i, group in enumerate(self.t_groups)}
         self.models_tau = {group: deepcopy(self.model_tau) for group in self.t_groups}
@@ -136,7 +119,7 @@ class BaseRLearner(object):
             self.vars_c[group] = (y_filt[w == 0] - yhat_filt[w == 0]).var()
             self.vars_t[group] = (y_filt[w == 1] - yhat_filt[w == 1]).var()
 
-    def predict(self, X):
+    def predict(self, X, p=None):
         """Predict treatment effects.
 
         Args:
@@ -177,17 +160,6 @@ class BaseRLearner(object):
         self.fit(X, treatment, y, p, verbose=verbose)
         te = self.predict(X)
 
-        if p is None:
-            p = self.propensity
-        else:
-            check_p_conditions(p, self.t_groups)
-
-        if isinstance(p, (np.ndarray, pd.Series)):
-            treatment_name = self.t_groups[0]
-            p = {treatment_name: convert_pd_to_np(p)}
-        elif isinstance(p, dict):
-            p = {treatment_name: convert_pd_to_np(_p) for treatment_name, _p in p.items()}
-
         if not return_ci:
             return te
         else:
@@ -199,6 +171,10 @@ class BaseRLearner(object):
 
             logger.info('Bootstrap Confidence Intervals')
             for i in tqdm(range(n_bootstraps)):
+                if p is None:
+                    p = self.propensity
+                else:
+                    p = self._format_p(p, self.t_groups)
                 te_b = self.bootstrap(X, treatment, y, p, size=bootstrap_size)
                 te_bootstraps[:, :, i] = te_b
 
@@ -230,18 +206,7 @@ class BaseRLearner(object):
             The mean and confidence interval (LB, UB) of the ATE estimate.
         """
         X, treatment, y = convert_pd_to_np(X, treatment, y)
-        te = self.fit_predict(X, treatment, y, p)
-
-        if p is None:
-            p = self.propensity
-        else:
-            check_p_conditions(p, self.t_groups)
-
-        if isinstance(p, (np.ndarray, pd.Series)):
-            treatment_name = self.t_groups[0]
-            p = {treatment_name: convert_pd_to_np(p)}
-        elif isinstance(p, dict):
-            p = {treatment_name: convert_pd_to_np(_p) for treatment_name, _p in p.items()}
+        te = self.fit_predict(X, treatment, y, p, return_ci=False)
 
         ate = np.zeros(self.t_groups.shape[0])
         ate_lb = np.zeros(self.t_groups.shape[0])
@@ -276,6 +241,10 @@ class BaseRLearner(object):
             ate_bootstraps = np.zeros(shape=(self.t_groups.shape[0], n_bootstraps))
 
             for n in tqdm(range(n_bootstraps)):
+                if p is None:
+                    p = self.propensity
+                else:
+                    p = self._format_p(p, self.t_groups)
                 cate_b = self.bootstrap(X, treatment, y, p, size=bootstrap_size)
                 ate_bootstraps[:, n] = cate_b.mean()
 
@@ -289,157 +258,6 @@ class BaseRLearner(object):
             self.models_tau = deepcopy(models_tau_global)
             return ate, ate_lower, ate_upper
 
-    def bootstrap(self, X, treatment, y, p, size=10000):
-        """Runs a single bootstrap. Fits on bootstrapped sample, then predicts on whole population."""
-
-        idxs = np.random.choice(np.arange(0, X.shape[0]), size=size)
-        X_b = X[idxs]
-        p_b = {group: _p[idxs] for group, _p in p.items()}
-        treatment_b = treatment[idxs]
-        y_b = y[idxs]
-        self.fit(X=X_b, treatment=treatment_b, y=y_b, p=p_b, verbose=False)
-        te_b = self.predict(X=X)
-        return te_b
-
-    def get_importance(self, X=None, tau=None, model_tau_feature=None, features=None, method='auto', normalize=True,
-                       test_size=0.3, random_state=None):
-        """
-        Builds a model (using X to predict estimated/actual tau), and then calculates feature importances
-        based on a specified method.
-
-        Currently supported methods include:
-            - auto (calculates importance based on estimator's default implementation of feature importance;
-                    estimator must be tree-based)
-                    Note: if none provided, it uses lightgbm's LGBMRegressor as estimator, and "gain" as
-                    importance type
-            - permutation (calculates importance based on mean decrease in accuracy when a feature column is permuted;
-                           estimator can be any form)
-        Hint: for permutation, downsample data for better performance especially if X.shape[1] is large
-
-        Args:
-            X (np.matrix or np.array or pd.Dataframe): a feature matrix
-            tau (np.array): a treatment effect vector (estimated/actual)
-            model_tau_feature (sklearn/lightgbm/xgboost model object): an unfitted model object
-            features (np.array): list/array of feature names. If None, an enumerated list will be used
-            method (str): auto, permutation
-            normalize (bool): normalize by sum of importances if method=gini (defaults to True)
-            test_size (float/int): if float, represents the proportion of the dataset to include in the test split.
-                                   If int, represents the absolute number of test samples (used for estimating
-                                   permutation importance)
-            random_state (int/RandomState instance/None): random state used in permutation importance estimation
-        """
-        explainer = Explainer(method=method, control_name=self.control_name,
-                              X=X, tau=tau, model_tau=model_tau_feature,
-                              features=features, classes=self._classes, normalize=normalize,
-                              test_size=test_size, random_state=random_state)
-        return explainer.get_importance()
-
-    def get_shap_values(self, X=None, model_tau_feature=None, tau=None, features=None):
-        """
-        Builds a model (using X to predict estimated/actual tau), and then calculates shapley values.
-        Args:
-            X (np.matrix or np.array or pd.Dataframe): a feature matrix
-            tau (np.array): a treatment effect vector (estimated/actual)
-            model_tau_feature (sklearn/lightgbm/xgboost model object): an unfitted model object
-            features (optional, np.array): list/array of feature names. If None, an enumerated list will be used.
-        """
-        explainer = Explainer(method='shapley', control_name=self.control_name,
-                              X=X, tau=tau, model_tau=model_tau_feature,
-                              features=features, classes=self._classes)
-        return explainer.get_shap_values()
-
-    def plot_importance(self, X=None, tau=None, model_tau_feature=None, features=None, method='auto', normalize=True,
-                        test_size=0.3, random_state=None):
-        """
-        Builds a model (using X to predict estimated/actual tau), and then plots feature importances
-        based on a specified method.
-
-        Currently supported methods include:
-            - auto (calculates importance based on estimator's default implementation of feature importance;
-                    estimator must be tree-based)
-                    Note: if none provided, it uses lightgbm's LGBMRegressor as estimator, and "gain" as
-                    importance type
-            - permutation (calculates importance based on mean decrease in accuracy when a feature column is permuted;
-                           estimator can be any form)
-        Hint: for permutation, downsample data for better performance especially if X.shape[1] is large
-
-        Args:
-            X (np.matrix or np.array or pd.Dataframe): a feature matrix
-            tau (np.array): a treatment effect vector (estimated/actual)
-            model_tau_feature (sklearn/lightgbm/xgboost model object): an unfitted model object
-            features (optional, np.array): list/array of feature names. If None, an enumerated list will be used
-            method (str): auto, permutation
-            normalize (bool): normalize by sum of importances if method=gini (defaults to True)
-            test_size (float/int): if float, represents the proportion of the dataset to include in the test split.
-                                   If int, represents the absolute number of test samples (used for estimating
-                                   permutation importance)
-            random_state (int/RandomState instance/None): random state used in permutation importance estimation
-        """
-        explainer = Explainer(method=method, control_name=self.control_name,
-                              X=X, tau=tau, model_tau=model_tau_feature,
-                              features=features, classes=self._classes, normalize=normalize,
-                              test_size=test_size, random_state=random_state)
-        explainer.plot_importance()
-
-    def plot_shap_values(self, X=None, tau=None, model_tau_feature=None, features=None, shap_dict=None, **kwargs):
-        """
-        Plots distribution of shapley values.
-
-        If shapley values have been pre-computed, pass it through the shap_dict parameter.
-        If shap_dict is not provided, this builds a new model (using X to predict estimated/actual tau),
-        and then calculates shapley values.
-
-        Args:
-            X (np.matrix or np.array or pd.Dataframe): a feature matrix. Required if shap_dict is None.
-            tau (np.array): a treatment effect vector (estimated/actual)
-            model_tau_feature (sklearn/lightgbm/xgboost model object): an unfitted model object
-            features (optional, np.array): list/array of feature names. If None, an enumerated list will be used.
-            shap_dict (optional, dict): a dict of shapley value matrices. If None, shap_dict will be computed.
-        """
-        override_checks = False if shap_dict is None else True
-        explainer = Explainer(method='shapley', control_name=self.control_name,
-                              X=X, tau=tau, model_tau=model_tau_feature,
-                              features=features, override_checks=override_checks, classes=self._classes)
-        explainer.plot_shap_values(shap_dict=shap_dict)
-
-    def plot_shap_dependence(self, treatment_group, feature_idx, X, tau, model_tau_feature=None, features=None,
-                             shap_dict=None, interaction_idx='auto', **kwargs):
-        """
-        Plots dependency of shapley values for a specified feature, colored by an interaction feature.
-
-        If shapley values have been pre-computed, pass it through the shap_dict parameter.
-        If shap_dict is not provided, this builds a new model (using X to predict estimated/actual tau),
-        and then calculates shapley values.
-
-        This plots the value of the feature on the x-axis and the SHAP value of the same feature
-        on the y-axis. This shows how the model depends on the given feature, and is like a
-        richer extension of the classical partial dependence plots. Vertical dispersion of the
-        data points represents interaction effects.
-
-        Args:
-            treatment_group (str or int): name of treatment group to create dependency plot on
-            feature_idx (str or int): feature index / name to create dependency plot on
-            X (np.matrix or np.array or pd.Dataframe): a feature matrix
-            tau (np.array): a treatment effect vector (estimated/actual)
-            model_tau_feature (sklearn/lightgbm/xgboost model object): an unfitted model object
-            features (optional, np.array): list/array of feature names. If None, an enumerated list will be used.
-            shap_dict (optional, dict): a dict of shapley value matrices. If None, shap_dict will be computed.
-            interaction_idx (optional, str or int): feature index / name used in coloring scheme as interaction feature.
-                If "auto" then shap.common.approximate_interactions is used to pick what seems to be the
-                strongest interaction (note that to find to true strongest interaction you need to compute
-                the SHAP interaction values).
-        """
-        override_checks = False if shap_dict is None else True
-        explainer = Explainer(method='shapley', control_name=self.control_name,
-                              X=X, tau=tau, model_tau=model_tau_feature,
-                              features=features, override_checks=override_checks,
-                              classes=self._classes)
-        explainer.plot_shap_dependence(treatment_group=treatment_group,
-                                       feature_idx=feature_idx,
-                                       shap_dict=shap_dict,
-                                       interaction_idx=interaction_idx,
-                                       **kwargs)
-
 
 class BaseRRegressor(BaseRLearner):
     """
@@ -450,6 +268,7 @@ class BaseRRegressor(BaseRLearner):
                  learner=None,
                  outcome_learner=None,
                  effect_learner=None,
+                 propensity_learner=ElasticNetPropensityModel(),
                  ate_alpha=.05,
                  control_name=0,
                  n_fold=5,
@@ -461,6 +280,8 @@ class BaseRRegressor(BaseRLearner):
             outcome_learner (optional): a model to estimate outcomes
             effect_learner (optional): a model to estimate treatment effects. It needs to take `sample_weight` as an
                 input argument for `fit()`
+            propensity_learner (optional): a model to estimate propensity scores. `ElasticNetPropensityModel()` will
+                be used by default.
             ate_alpha (float, optional): the confidence level alpha of the ATE estimate
             control_name (str or int, optional): name of control group
             n_fold (int, optional): the number of cross validation folds for outcome_learner
@@ -470,6 +291,7 @@ class BaseRRegressor(BaseRLearner):
             learner=learner,
             outcome_learner=outcome_learner,
             effect_learner=effect_learner,
+            propensity_learner=propensity_learner,
             ate_alpha=ate_alpha,
             control_name=control_name,
             n_fold=n_fold,
@@ -484,6 +306,7 @@ class BaseRClassifier(BaseRLearner):
     def __init__(self,
                  outcome_learner=None,
                  effect_learner=None,
+                 propensity_learner=ElasticNetPropensityModel(),
                  ate_alpha=.05,
                  control_name=0,
                  n_fold=5,
@@ -494,6 +317,8 @@ class BaseRClassifier(BaseRLearner):
             outcome_learner: a model to estimate outcomes. Should be a classifier.
             effect_learner: a model to estimate treatment effects. It needs to take `sample_weight` as an
                 input argument for `fit()`. Should be a regressor.
+            propensity_learner (optional): a model to estimate propensity scores. `ElasticNetPropensityModel()` will
+                be used by default.
             ate_alpha (float, optional): the confidence level alpha of the ATE estimate
             control_name (str or int, optional): name of control group
             n_fold (int, optional): the number of cross validation folds for outcome_learner
@@ -503,6 +328,7 @@ class BaseRClassifier(BaseRLearner):
             learner=None,
             outcome_learner=outcome_learner,
             effect_learner=effect_learner,
+            propensity_learner=propensity_learner,
             ate_alpha=ate_alpha,
             control_name=control_name,
             n_fold=n_fold,
@@ -529,27 +355,10 @@ class BaseRClassifier(BaseRLearner):
         self.t_groups.sort()
 
         if p is None:
-            logger.info('Generating propensity score')
-            p = dict()
-            p_model = dict()
-            for group in self.t_groups:
-                mask = (treatment == group) | (treatment == self.control_name)
-                treatment_filt = treatment[mask]
-                X_filt = X[mask]
-                w_filt = (treatment_filt == group).astype(int)
-                w = (treatment == group).astype(int)
-                p[group], p_model[group] = compute_propensity_score(X=X_filt, treatment=w_filt,
-                                                                    X_pred=X, treatment_pred=w)
-            self.propensity_model = p_model
-            self.propensity = p
+            self._set_propensity_models(X=X, treatment=treatment, y=y)
+            p = self.propensity
         else:
-            check_p_conditions(p, self.t_groups)
-
-        if isinstance(p, (np.ndarray, pd.Series)):
-            treatment_name = self.t_groups[0]
-            p = {treatment_name: convert_pd_to_np(p)}
-        elif isinstance(p, dict):
-            p = {treatment_name: convert_pd_to_np(_p) for treatment_name, _p in p.items()}
+            p = self._format_p(p, self.t_groups)
 
         self._classes = {group: i for i, group in enumerate(self.t_groups)}
         self.models_tau = {group: deepcopy(self.model_tau) for group in self.t_groups}
@@ -577,7 +386,7 @@ class BaseRClassifier(BaseRLearner):
             self.vars_c[group] = (y_filt[w == 0] - yhat_filt[w == 0]).var()
             self.vars_t[group] = (y_filt[w == 1] - yhat_filt[w == 1]).var()
 
-    def predict(self, X):
+    def predict(self, X, p=None):
         """Predict treatment effects.
 
         Args:
@@ -654,27 +463,10 @@ class XGBRRegressor(BaseRRegressor):
         self.t_groups.sort()
 
         if p is None:
-            logger.info('Generating propensity score')
-            p = dict()
-            p_model = dict()
-            for group in self.t_groups:
-                mask = (treatment == group) | (treatment == self.control_name)
-                treatment_filt = treatment[mask]
-                X_filt = X[mask]
-                w_filt = (treatment_filt == group).astype(int)
-                w = (treatment == group).astype(int)
-                p[group], p_model[group] = compute_propensity_score(X=X_filt, treatment=w_filt,
-                                                                    X_pred=X, treatment_pred=w)
-            self.propensity_model = p_model
-            self.propensity = p
+            self._set_propensity_models(X=X, treatment=treatment, y=y)
+            p = self.propensity
         else:
-            check_p_conditions(p, self.t_groups)
-
-        if isinstance(p, (np.ndarray, pd.Series)):
-            treatment_name = self.t_groups[0]
-            p = {treatment_name: convert_pd_to_np(p)}
-        elif isinstance(p, dict):
-            p = {treatment_name: convert_pd_to_np(_p) for treatment_name, _p in p.items()}
+            p = self._format_p(p, self.t_groups)
 
         self._classes = {group: i for i, group in enumerate(self.t_groups)}
         self.models_tau = {group: deepcopy(self.model_tau) for group in self.t_groups}
