@@ -14,14 +14,19 @@ The module structure is the following:
 # Authors: Zhenyu Zhao <zhenyuz@uber.com>
 #          Totte Harinen <totte@uber.com>
 
-from collections import defaultdict
-import numpy as np
-import scipy.stats as stats
-import pandas as pd
-from sklearn.utils.testing import ignore_warnings
 from collections import defaultdict
 from joblib import Parallel, delayed
 import multiprocessing as mp
+import numpy as np
+from packaging import version
+import pandas as pd
+import scipy.stats as stats
+import sklearn
+if version.parse(sklearn.__version__) >= version.parse('0.22.0'):
+    from sklearn.utils._testing import ignore_warnings
+else:
+    from sklearn.utils.testing import ignore_warnings
+
 
 class DecisionTree:
     """ Tree Node Class
@@ -500,7 +505,8 @@ class UpliftTreeClassifier:
         else:
             return treatment_optimal, p_hat_optimal
 
-    def divideSet(self, X, treatment, y, column, value):
+    @staticmethod
+    def divideSet(X, treatment, y, column, value):
         '''
         Tree node split.
 
@@ -1052,7 +1058,8 @@ class UpliftTreeClassifier:
                     bestTreatment=bestTreatment, upliftScore=upliftScore
                 )
 
-    def classify(self, observations, tree, dataMissing=False):
+    @staticmethod
+    def classify(observations, tree, dataMissing=False):
         '''
         Classifies (prediction) the observations according to the tree.
 
@@ -1194,6 +1201,10 @@ class UpliftRandomForestClassifier:
         The normalization factor defined in Rzepakowski et al. 2012,
         correcting for tests with large number of splits and imbalanced
         treatment and control splits
+    
+    n_jobs: int, optional (default=-1)
+        The parallelization parameter to define how many parallel jobs need to be created. 
+        This is passed on to joblib library for parallelizing uplift-tree creation.
 
     Outputs
     ----------
@@ -1278,7 +1289,8 @@ class UpliftRandomForestClassifier:
         self.feature_importances_ = np.mean(all_importances, axis=0)
         self.feature_importances_ /= self.feature_importances_.sum()  # normalize to add to 1
 
-    def bootstrap(self, X, treatment, y, tree):
+    @staticmethod
+    def bootstrap(X, treatment, y, tree):
         bt_index = np.random.choice(len(X), len(X))
         x_train_bt = X[bt_index]
         y_train_bt = y[bt_index]
